/*-
 * %%----------------------------------------------------------------------------------------------
 * Solidify Framework - Solidify Frontend - array.util.ts
 * SPDX-License-Identifier: GPL-2.0-or-later
 * %----------------------------------------------------------------------------------------------%
 * Copyright (C) 2017 - 2023 University of Geneva
 * %----------------------------------------------------------------------------------------------%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/gpl-2.0.html>.
 * ----------------------------------------------------------------------------------------------%%
 */

import _ from "lodash";
import {
  isArray,
  isFunction,
  isNullOrUndefined,
} from "../tools/is/is.tool";

export class ArrayUtil {
  static distinct<T>(array: T[], key: string | undefined = undefined): T[] {
    const result = [];
    const map = new Map();
    for (const item of array) {
      let mapKey = item;
      if (!isNullOrUndefined(key)) {
        mapKey = item[key];
      }
      if (!map.has(mapKey)) {
        map.set(mapKey, item);
        result.push(item);
      }
    }
    return result;
  }

  /**
   * Allow to get diff between 2 array
   *
   * @param arrayA
   * @param arrayB
   */
  static diff<T>(arrayA: T[], arrayB: T[], comparator: (a: T, b, T) => boolean = undefined): ArrayDiffResult<T> {
    /* eslint-disable no-console */
    if (isNullOrUndefined(arrayA)) {
      console.warn("'arrayA' param should not be null");
      return undefined;
    }
    if (!isArray(arrayA)) {
      console.warn("'arrayA' param should be an array");
      return undefined;
    }
    if (isNullOrUndefined(arrayB)) {
      console.warn("'arrayB' param should not be null");
      return undefined;
    }
    if (!isArray(arrayB)) {
      console.warn("'arrayB' param should be an array");
      return undefined;
    }
    /* eslint-enable no-console */
    const arrayDiffResult = {
      arrayA: arrayA,
      arrayB: arrayB,
    } as ArrayDiffResult<T>;
    if (isFunction(comparator)) {
      arrayDiffResult.diff = _.xorWith(arrayA, arrayB, comparator);
      arrayDiffResult.common = _.intersectionWith(arrayA, arrayB, comparator);
      arrayDiffResult.presentOnlyInArrayA = _.intersectionWith(arrayA, arrayDiffResult.diff, comparator);
      arrayDiffResult.presentOnlyInArrayB = _.intersectionWith(arrayB, arrayDiffResult.diff, comparator);
    } else {
      arrayDiffResult.diff = _.xor(arrayA, arrayB);
      arrayDiffResult.common = _.intersection(arrayA, arrayB);
      arrayDiffResult.presentOnlyInArrayA = _.intersection(arrayA, arrayDiffResult.diff);
      arrayDiffResult.presentOnlyInArrayB = _.intersection(arrayB, arrayDiffResult.diff);
    }
    arrayDiffResult.missingInArrayA = arrayDiffResult.presentOnlyInArrayB;
    arrayDiffResult.missingInArrayB = arrayDiffResult.presentOnlyInArrayA;
    return arrayDiffResult;
  }

  static insert<T>(array: T[], index: number, ...items: T[]): T[] {
    /* eslint-disable no-console */
    if (isNullOrUndefined(array)) {
      console.warn("'array' param should not be null");
      return undefined;
    }
    if (!isArray(array)) {
      console.warn("'array' param should be an array");
      return undefined;
    }
    if (isNullOrUndefined(items)) {
      console.warn("'items' param should not be null");
      return undefined;
    }
    if (!isArray(items)) {
      console.warn("'items' param should be an array");
      return undefined;
    }
    if (index > array.length) {
      console.warn("'index' is out of bound");
      return undefined;
    }
    /* eslint-enable no-console */
    array = [...array];
    array.splice(index, 0, ...items);
    return array;
  }
}

export interface ArrayDiffResult<T> {
  arrayA: T[];
  arrayB: T[];
  diff: T[];
  common: T[];
  missingInArrayA: T[];
  missingInArrayB: T[];
  presentOnlyInArrayA: T[];
  presentOnlyInArrayB: T[];
}
